@@ -1,4 +1,5 @@
-from urllib.parse import quote_plus, urljoin
+from datetime import datetime
+from urllib.parse import urljoin
 
 from django.conf import settings
 
@@ -7,6 +8,8 @@ from terminusgps.wialon.items.base import WialonBase
 
 
 class WialonUnit(WialonBase):
+    """A Wialon `unit <https://help.wialon.com/en/wialon-hosting/user-guide/management-system/units>`_."""
+
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self._imei_number = None
@@ -19,15 +22,16 @@ class WialonUnit(WialonBase):
         """
         Creates a new Wialon unit.
 
-        :param creator_id: A Wialon user id.
+        :param creator_id: Creator user id.
         :type creator_id: :py:obj:`str` | :py:obj:`int`
-        :param name: A name for the unit.
+        :param name: A name for the new unit.
         :type name: :py:obj:`str`
-        :param hw_type_id: A Wialon hardware type ID.
+        :param hw_type_id: A Wialon hardware type id for the new unit.
         :type hw_type_id: :py:obj:`str` | :py:obj:`int`
-        :raises ValueError: If ``creator_id`` is a :py:obj:`str` but not a digit.
-        :raises ValueError: If ``hw_type_id`` is a :py:obj:`str` but not a digit.
-        :returns: An id for the new unit.
+        :raises ValueError: If ``creator_id`` isn't a digit.
+        :raises ValueError: If ``hw_type_id`` isnt' a digit.
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: A new Wialon unit id, if created.
         :rtype: :py:obj:`int` | :py:obj:`None`
 
         """
@@ -38,9 +42,9 @@ class WialonUnit(WialonBase):
 
         response = self.session.wialon_api.core_create_unit(
             **{
-                "creatorId": creator_id,
-                "name": name,
-                "hwTypeId": hw_type_id,
+                "creatorId": str(creator_id),
+                "name": str(name),
+                "hwTypeId": str(hw_type_id),
                 "dataFlags": flags.DataFlag.UNIT_BASE,
             }
         )
@@ -51,6 +55,7 @@ class WialonUnit(WialonBase):
         )
 
     def populate(self) -> None:
+        """Sets :py:attr:`imei_number`, :py:attr:`active` and :py:attr:`image_uri`."""
         super().populate()
         response = self.session.wialon_api.core_search_item(
             **{
@@ -68,22 +73,203 @@ class WialonUnit(WialonBase):
             self._active = item.get("act", False)
             self._image_uri = item.get("uri")
 
-    def get_position(self) -> dict | None:
-        return self.session.wialon_api.core_search_item(
+    def get_position(self) -> dict:
+        """
+        Returns the current position of the unit.
+
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: The unit's current position.
+        :rtype: :py:obj:`dict`
+
+        Response format:
+
+        +----------+--------------------+-----------------+
+        | key      | type               | desc            |
+        +==========+====================+=================+
+        | ``"t"``  | :py:obj:`datetime` | Time (UTC)      |
+        +----------+--------------------+-----------------+
+        | ``"y"``  | :py:obj:`float`    | Latitude        |
+        +----------+--------------------+-----------------+
+        | ``"x"``  | :py:obj:`float`    | Longitude       |
+        +----------+--------------------+-----------------+
+        | ``"z"``  | :py:obj:`float`    | Altitude        |
+        +----------+--------------------+-----------------+
+        | ``"s"``  | :py:obj:`int`      | Speed           |
+        +----------+--------------------+-----------------+
+        | ``"c"``  | :py:obj:`int`      | Course          |
+        +----------+--------------------+-----------------+
+        | ``"sc"`` | :py:obj:`int`      | # of satellites |
+        +----------+--------------------+-----------------+
+
+        """
+        response = self.session.wialon_api.core_search_item(
             **{"id": self.id, "flags": flags.DataFlag.UNIT_POSITION}
+        ).get("pos", {})
+        if response:
+            response["t"] = datetime.fromtimestamp(response["t"])
+        return response
+
+    def execute_command(
+        self,
+        name: str,
+        link_type: str,
+        timeout: int = 5,
+        flags: int = 0,
+        param: dict | None = None,
+    ) -> None:
+        """
+        Executes a command on the unit.
+
+        :param name: A Wialon command name.
+        :type name: :py:obj:`str`
+        :param link_type: A protocol to use for the Wialon command.
+        :type link_type: :py:obj:`str`
+        :param timeout: How long (in seconds) to wait before timing out command execution. Default is ``5``.
+        :type timeout: :py:obj:`int`
+        :param flags: Flags to pass to the Wialon command execution.
+        :type flags: :py:obj:`int`
+        :param param: Additional parameters to execute the command with.
+        :type param: :py:obj:`dict` | :py:obj:`None`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.unit_exec_cmd(
+            **{
+                "itemId": self.id,
+                "commandName": name,
+                "linkType": link_type,
+                "timeout": timeout,
+                "flags": flags,
+                "param": param if param else {},
+            }
         )
 
-    @property
-    def position(self) -> dict | None:
-        """Current GPS position of the unit."""
-        return self.get_position()
+    def set_access_password(self, password: str) -> None:
+        """
+        Sets the unit's access password.
+
+        :param password: A new access password.
+        :type password: :py:obj:`str`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.unit_update_access_password(
+            **{"itemId": self.id, "accessPassword": password}
+        )
+
+    def activate(self) -> None:
+        """
+        Activates the unit.
+
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.unit_set_active(
+            **{"itemId": self.id, "active": int(True)}
+        )
+
+    def deactivate(self) -> None:
+        """
+        Deactivates the unit.
+
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.unit_set_active(
+            **{"itemId": self.id, "active": int(False)}
+        )
+
+    def get_phone_numbers(
+        self, cfield_key: str = "to_number", afield_key: str = "to_number"
+    ) -> list[str]:
+        """
+        Retrieves all phone numbers assigned to the unit.
+
+        This includes any attached drivers, custom/admin fields or otherwise assigned phone numbers.
+
+        :param cfield_key: A custom field key used to retrieve phone numbers. Default is :py:obj:`"to_number"`.
+        :type cfield_key: :py:obj:`str`
+        :param afield_key: An admin field key used to retrieve phone numbers. Default is :py:obj:`"to_number"`.
+        :type afield_key: :py:obj:`str`
+        :raises WialonError: If something goes wrong with Wialon.
+        :returns: A list of phone numbers.
+        :rtype: :py:obj:`list`
+
+        """
+        phones_0: list[str] = self._get_driver_phone_numbers()
+        phones_1: list[str] = self._get_cfield_phone_numbers(key=cfield_key)
+        phones_2: list[str] = self._get_afield_phone_numbers(key=afield_key)
+        return list(frozenset(phones_0 + phones_1 + phones_2))
+
+    def clean_phone_numbers(self, phones: list[str]) -> list[str]:
+        """Takes a list of phone numbers and returns a list of clean phone numbers."""
+        return [
+            clean_num
+            for num in phones
+            for clean_num in (num.split(",") if "," in num else [num])
+        ]
+
+    def _get_afield_phone_numbers(self, key: str) -> list[str]:
+        """
+        Retrives phone numbers saved in an admin field by key.
+
+        :param key: An admin field key.
+        :type key: :py:obj:`str`
+        :raises WialonError: If something goes wrong with Wialon.
+        :returns: A list of phone numbers.
+        :rtype: :py:obj:`list`
+
+        """
+        if key not in self.afields.keys():
+            return []
+        return self.clean_phone_numbers([self.afields[key]])
+
+    def _get_cfield_phone_numbers(self, key: str) -> list[str]:
+        """
+        Retrives phone numbers saved in a custom field by key.
+
+        :param key: A custom field key.
+        :type key: :py:obj:`str`
+        :raises WialonError: If something goes wrong with Wialon.
+        :returns: A list of phone numbers.
+        :rtype: :py:obj:`list`
+
+        """
+        if key not in self.cfields.keys():
+            return []
+        return self.clean_phone_numbers([self.cfields[key]])
+
+    def _get_driver_phone_numbers(self) -> list[str]:
+        """
+        Returns a list of phone numbers assigned to drivers attached to the unit.
+
+        :raises WialonError: If something goes wrong with Wialon.
+        :returns: A list of phone numbers.
+        :rtype: :py:obj:`list`
+
+        """
+        response = self.session.wialon_api.resource_get_unit_drivers(
+            **{"unitId": self.id}
+        )
+        if response:
+            dirty_phones = [driver[0].get("ph") for driver in response.values()]
+            return self.clean_phone_numbers(dirty_phones)
+        return []
 
     @property
     def exists(self) -> bool:
         """Whether or not the unit exists in Wialon."""
         return bool(
             self.session.wialon_api.core_search_item(
-                **{"id": self.id, "flags": flags.DataFlag.UNIT_BASE.value}
+                **{"id": self.id, "flags": flags.DataFlag.UNIT_BASE}
             ).get("item", False)
         )
 
@@ -153,227 +339,6 @@ class WialonUnit(WialonBase):
     @property
     def image_url(self) -> str | None:
         """Returns an absolute url to the unit's icon in Wialon."""
-        if hasattr(settings, "WIALON_HOST"):
+        if settings.configured and hasattr(settings, "WIALON_HOST"):
             return urljoin(settings.WIALON_HOST, self._image_uri)
         return urljoin("https://hst-api.wialon.com", self._image_uri)
-
-    def assign_to_number(self, value: str) -> None:
-        """
-        Sets ``value`` to the unit's ``to_number`` custom field.
-
-        :param value: A new value for the unit's ``to_number`` custom field.
-        :type value: :py:obj:`str`
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        cfield_key: str = "to_number"
-        cfield_val: str = value
-        cfield_id: int | None = self.get_cfield_id(cfield_key)
-
-        if cfield_id:
-            self.update_cfield(cfield_id, cfield_key, cfield_val)
-
-    def assign_iccid(self, value: str) -> None:
-        """
-        Sets ``value`` to the unit's ``iccid`` admin field.
-
-        :param value: A new value for the unit's ``iccid`` custom field.
-        :type value: :py:obj:`str`
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        afield_key: str = "iccid"
-        afield_val: str = value
-        afield_id: int | None = self.get_afield_id(afield_key)
-
-        if afield_id:
-            self.update_afield(afield_id, afield_key, afield_val)
-
-    def assign_carrier(self, value: str) -> None:
-        """
-        Sets ``value`` to the unit's ``carrier`` admin field.
-
-        :param value: A new value for the unit's ``carrier`` custom field.
-        :type value: :py:obj:`str`
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        afield_key: str = "carrier"
-        afield_val: str = value
-        afield_id: int | None = self.get_afield_id(afield_key)
-
-        if afield_id:
-            self.update_afield(afield_id, afield_key, afield_val)
-
-    def execute_command(
-        self,
-        name: str,
-        link_type: str,
-        timeout: int = 5,
-        flags: int = 0,
-        param: dict | None = None,
-    ) -> None:
-        """
-        Executes a command on the unit.
-
-        :param name: A Wialon command name.
-        :type name: :py:obj:`str`
-        :param link_type: A protocol to use for the Wialon command.
-        :type link_type: :py:obj:`str`
-        :param timeout: How long (in seconds) to wait before timing out command execution. Default is ``5``.
-        :type timeout: :py:obj:`int`
-        :param flags: Flags to pass to the Wialon command execution.
-        :type flags: :py:obj:`int`
-        :param param: Additional parameters to execute the command with.
-        :type param: :py:obj:`dict` | :py:obj:`None`
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.unit_exec_cmd(
-            **{
-                "itemId": self.id,
-                "commandName": name,
-                "linkType": link_type,
-                "timeout": timeout,
-                "flags": flags,
-                "param": param if param else {},
-            }
-        )
-
-    def set_access_password(self, password: str) -> None:
-        """
-        Sets a new access password for the unit.
-
-        :param password: A new access password.
-        :type name: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.unit_update_access_password(
-            **{"itemId": self.id, "accessPassword": password}
-        )
-
-    def activate(self) -> None:
-        """
-        Activates the unit.
-
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.unit_set_active(
-            **{"itemId": self.id, "active": int(True)}
-        )
-
-    def deactivate(self) -> None:
-        """
-        Deactivates the unit.
-
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.unit_set_active(
-            **{"itemId": self.id, "active": int(False)}
-        )
-
-    def assign_phone(self, phone: str) -> None:
-        """
-        Assigns a phone number to the unit.
-
-        :param phone: A phone number beginning with a country code.
-        :type phone: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.unit_update_phone(
-            **{"itemId": self.id, "phoneNumber": quote_plus(phone)}
-        )
-
-    def get_phone_numbers(self) -> list[str]:
-        """
-        Retrieves all phone numbers assigned to the unit.
-
-        This includes any attached drivers, custom/admin fields or otherwise assigned phone numbers.
-
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: A list of phone numbers.
-        :rtype: :py:obj:`list`
-
-        """
-        phone_numbers: list[str] = []
-        phones_0: list[str] | None = self._get_driver_phone_numbers()
-        phones_1: list[str] | None = self._get_cfield_phone_numbers()
-        phones_2: list[str] | None = self._get_afield_phone_numbers()
-
-        if phones_0 is not None:
-            phone_numbers.extend(phones_0)
-        if phones_1 is not None:
-            phone_numbers.extend(phones_1)
-        if phones_2 is not None:
-            phone_numbers.extend(phones_2)
-        return list(dict.fromkeys(phone_numbers))  # Removes duplicate phone numbers
-
-    def clean_phone_numbers(self, phones: list[str]) -> list[str]:
-        """Takes a list of phone numbers and returns a list of clean phone numbers."""
-        return [
-            clean_num
-            for num in phones
-            for clean_num in (num.split(",") if "," in num else [num])
-        ]
-
-    def _get_afield_phone_numbers(self, key: str = "to_number") -> list[str] | None:
-        """
-        Retrives any phone numbers saved in an admin field by key.
-
-        :param key: An admin field key. Default is ``"to_number"``.
-        :type key: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: A list of phone numbers, if the unit has admin phone number fields.
-        :rtype: :py:obj:`list` | :py:obj:`None`
-
-        """
-        for field_name, field_val in self.afields.items():
-            if field_name == key:
-                return self.clean_phone_numbers([field_val])
-
-    def _get_cfield_phone_numbers(self, key: str = "to_number") -> list[str] | None:
-        """
-        Retrives any phone numbers saved in a custom field by key.
-
-        :param key: A custom field key. Default is ``"to_number"``.
-        :type key: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: A list of phone numbers, if the unit has custom phone number fields.
-        :rtype: :py:obj:`list` | :py:obj:`None`
-
-        """
-        for field_name, field_val in self.cfields.items():
-            if field_name == key:
-                return self.clean_phone_numbers([field_val])
-
-    def _get_driver_phone_numbers(self) -> list[str] | None:
-        """
-        Retrieves any phone numbers assigned to drivers attached to the unit.
-
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: A list of phone numbers, if the unit has attached drivers with phone numbers.
-        :rtype: :py:obj:`list` | :py:obj:`None`
-
-        """
-        response = self.session.wialon_api.resource_get_unit_drivers(
-            **{"unitId": self.id}
-        )
-        if response:
-            dirty_phones = [driver[0].get("ph") for driver in response.values()]
-            return self.clean_phone_numbers(dirty_phones)
